/****************************************************************************
** menu.h file.
**
** This file handles the user input to generate a DVD menu.
** Please note that this is work in progress and depends also oin the
** developement of dvdauthor, which is at the moment in version 0.67.
** Future versions of dvdauthort might need changes to this file.
**
*****************************************************************************/
//#include <iostream>

#include <qmessagebox.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qdom.h>

#include "global.h"
#include "qdvdauthor.h"
#include "dvdmenu.h"
#include "dialogresize.h"
#include "xml_dvd.h"

//////////////////////////////////////////////////////////////
//
// The Interface class ...
//
//////////////////////////////////////////////////////////////

CDVDMenuInterface::CDVDMenuInterface ()
{
	bModified = false;
	qsBackgroundFileName = QString ("");
	iFormat = FORMAT_NONE;
	iWidth = 0;
	iHeight = 0;
}

CDVDMenuInterface::~CDVDMenuInterface ()
{

}

///////////////////////////////////////////////
//
// This function loads the xml project file
// specified by the  user.
//
// The file specs are defined in xml_dvd.h
// Note : I chose to use the same syntax as
// dvdauthor and spumux uses only incorporated
// into one single xml file which also has
// all other project settings.
//
////////////////////////////////////////////////
bool CDVDMenuInterface::readXml (QDomDocument *pXmlDocument)
{
	QDomElement docElem = pXmlDocument->documentElement();

//printf ("<%s>\n", (const char *)docElem.tagName ());
	// The First child holds the <QDVDProject> - root node.
	QDomNode xmlNode = docElem.firstChild();
//printf ("<%s><%s><%s><%s>\n",(const char *) n.localName (), (const char *)n.nodeValue(), (const char *)n.namespaceURI(), (const char *)n.prefix());

	while( !xmlNode.isNull() ) {
		QDomElement searchDVDMenu = xmlNode.toElement();
		if ( DVD_MENU == searchDVDMenu.tagName () )	{
			// Here we get the DVDMenu Nodes ...
			QDomNode xmlDVDMenu = searchDVDMenu.firstChild();
			printf ("I found it <%s>\n", (const char *)searchDVDMenu.tagName());
			while ( !xmlDVDMenu.isNull () )	{
				// Okay, here we retain the stored data from the xml file.
				extractXml (&xmlDVDMenu);
				// So lets get the next sibling ... until we hit hte end of DVDMenu ...
				xmlDVDMenu = xmlDVDMenu.nextSibling();
			}
			break;
		}
		else
			printf ("Nah, this is ... <%s>\n", (const char *)searchDVDMenu.tagName());
		// Go to the next node ...
		xmlNode = xmlNode.nextSibling();
	}
	bModified = false;
	return true;
}

void CDVDMenuInterface::extractXml (QDomNode *pNode)
{
//	char [50][] = {"", ""};
	QDomElement theElement = pNode->toElement();
	QString tagName = theElement.tagName();
	QString nodeText = theElement.text ();
	printf ("<%s><%s>\n", (const char *)tagName, (const char *)nodeText);
	// Okay, this is ugly but after all it is the simplest of all xml file structure.
	// No need to get fancy ...
	if (tagName == MENU_BACKGROUND)
		qsBackgroundFileName = nodeText;
	else if (tagName == MENU_WIDTH)
		iWidth = nodeText.toInt();
	else if (tagName == MENU_HEIGHT)
		iHeight = nodeText.toInt();
}

void CDVDMenuInterface::createXml (QDomDocument *pXmlDocument)
{
	QDomElement rootElement = pXmlDocument->documentElement();

	QDomElement rootDVDMenu = pXmlDocument->createElement( DVD_MENU );
	rootElement.appendChild( rootDVDMenu );

		QDomElement tag = pXmlDocument->createElement( MENU_BACKGROUND );
		rootDVDMenu.appendChild( tag );

		QDomText t = pXmlDocument->createTextNode(qsBackgroundFileName);
		tag.appendChild( t );

//		tag = pXmlDocument->createElement( "FORMAT" );
//		rootDVDMenu.appendChild( tag );
//		tag.setAttribute( MENU_WIDTH, QString("%1").arg(iWidth) );
//		tag.setAttribute( MENU_HEIGHT, QString("%1").arg(iHeight) );

		tag = pXmlDocument->createElement( MENU_WIDTH );
		rootDVDMenu.appendChild( tag );

		t = pXmlDocument->createTextNode(QString("%1").arg(iWidth) );
		tag.appendChild( t );

		tag = pXmlDocument->createElement( MENU_HEIGHT );
		rootDVDMenu.appendChild( tag );

		t = pXmlDocument->createTextNode(QString("%1").arg(iHeight) );
		tag.appendChild( t );
}


//////////////////////////////////////////////////////////////
//
// The Main class
//
//////////////////////////////////////////////////////////////

CDVDMenu::CDVDMenu ()
{

}

CDVDMenu::CDVDMenu (QDVDAuthor *pDVDAuthor)
{
	CreateTab 			(pDVDAuthor->m_pTabWidgetMain);
	CreateToolBar		(pDVDAuthor);
	CreateMenuEntry	(pDVDAuthor);
}

CDVDMenu::~CDVDMenu ()
{

}

// Standard initialization Functions.
void CDVDMenu::CreateTab (QTabWidget *pTabWidgetMain)
{
    m_pDVDMenuTab = new QWidget( pTabWidgetMain, "m_pDVDMenuTab" );
    m_pTabLayout = new QGridLayout( m_pDVDMenuTab, 1, 1, 11, 6, "m_pTabLayout");

    m_pLayout1 = new QGridLayout( 0, 1, 1, 0, 6, "m_pLayout1");
    m_pLayout2 = new QHBoxLayout( 0, 0, 6, "m_pLayout2");

    m_pPixmapMenu = new QLabel (m_pDVDMenuTab, "m_pPixmapMenu" );
    m_pPixmapMenu->setFrameShape( QLabel::Box );
    m_pPixmapMenu->setFrameShadow( QLabel::Raised );
    m_pPixmapMenu->setLineWidth( 2 );
    m_pPixmapMenu->setMargin( 0 );
    m_pPixmapMenu->setScaledContents( TRUE );
	m_pPixmapMenu->setBackgroundColor (QColor (0, 0, 0));

    m_pLayout1->addWidget( m_pPixmapMenu, 0, 0 );

    m_pButtonAddBackground = new QPushButton( m_pDVDMenuTab, "m_pButtonAddBackground" );
    m_pLayout2->addWidget( m_pButtonAddBackground );

    m_pButtonAddSound = new QPushButton( m_pDVDMenuTab, "m_pButtonAddSound" );
    m_pLayout2->addWidget( m_pButtonAddSound );

    m_pButtonAddMask = new QPushButton( m_pDVDMenuTab, "m_pButtonAddMask" );
    m_pLayout2->addWidget( m_pButtonAddMask );

    m_pButtonDrawMask = new QPushButton( m_pDVDMenuTab, "m_pButtonDrawMask" );
    m_pLayout2->addWidget( m_pButtonDrawMask );

    m_pLayout1->addLayout( m_pLayout2, 1, 0 );

    m_pTabLayout->addLayout( m_pLayout1, 0, 0 );
    pTabWidgetMain->insertTab( m_pDVDMenuTab, tr( "Titleset (Menu)" ), 0);

    m_pButtonAddBackground->setText( tr( "Add Background" ) );
    m_pButtonAddSound->setText( tr( "Add Sound" ) );
    m_pButtonAddMask->setText( tr( "Add Mask" ) );
    m_pButtonDrawMask->setText( tr( "Draw Mask" ) );
//    pTabWidgetMain->changeTab( m_pDVDMenuTab, tr( "Titleset (Menu)" ) );
	pTabWidgetMain->setCurrentPage (0);

	connect( m_pButtonAddBackground, SIGNAL( clicked() ), this, SLOT( slotAddBackground() ) );
	connect( m_pButtonAddMask, SIGNAL( clicked() ), this, SLOT( slotAddMask() ) );
	connect( m_pButtonAddSound, SIGNAL( clicked() ), this, SLOT( slotAddSound() ) );
	connect( m_pButtonDrawMask, SIGNAL( clicked() ), this, SLOT( slotDrawMask() ) );
}

void CDVDMenu::CreateToolBar (QDVDAuthor *pDVDAuthor)
{
	// Create some action ...
	m_pActionZoomIn  = new QAction( pDVDAuthor, "ZoomIn" );
	m_pActionZoomOut = new QAction( pDVDAuthor, "ZoomOut" );
	m_pActionZoomIn->setIconSet( QIconSet( QPixmap( "filenew" ) ) );
//	m_pActionZoomIn->setIconSet( QIconSet( QPixmap::fromMimeSource( "filenew" ) ) );
	m_pActionAbout  = new QAction( pDVDAuthor, "helpAboutAction" );

	m_pActionZoomIn->setMenuText( tr( "Zoom &In" ) );
	m_pActionZoomIn->setAccel( QString::null );
	m_pActionZoomOut->setMenuText( tr( "Zoom &Out" ) );
	m_pActionZoomOut->setAccel( QString::null );
	m_pActionAbout->setMenuText( tr( "&About" ) );
	m_pActionAbout->setAccel( QString::null );

	// Here we create the ToolBar
	m_pToolBar = new QToolBar( "", pDVDAuthor, DockTop );

	m_pActionZoomIn->addTo( m_pToolBar );
	m_pActionZoomOut->addTo( m_pToolBar );
	m_pToolBar->addSeparator();
	m_pActionAbout->addTo( m_pToolBar );
	m_pToolBar->addSeparator();
}

void CDVDMenu::CreateMenuEntry (QDVDAuthor *pDVDAuthor)
{
	// menubar
	m_pDVDMenuMenu = new QPopupMenu( pDVDAuthor);
	m_pActionZoomIn->addTo( m_pDVDMenuMenu );
	m_pActionZoomOut->addTo( m_pDVDMenuMenu );
	m_pActionAbout->addTo( m_pDVDMenuMenu );
	pDVDAuthor->menubar->insertItem( "&DVDMenu", m_pDVDMenuMenu, 0, 2 );

	// signals and slots connections
	connect( m_pActionZoomIn,		SIGNAL( activated() ), this, SLOT( slotZoomIn() ) );
	connect( m_pActionZoomOut,	SIGNAL( activated() ), this, SLOT( slotZoomOut() ) );
	connect( m_pActionAbout,		SIGNAL( activated() ), this, SLOT( slotAbout() ) );
}

// Public slots. Here are the main actors from the GUI.
void CDVDMenu::slotAddBackground()
{

	QString fileName = QFileDialog::getOpenFileName ( QString("./"), QString ("Images (*.png *.xpm *.jpg)"));

	QPixmap theBackground = QPixmap (fileName);
	// Now we check if the image has already the right format for PAL / NTSC / SECAM
	int iHeight = theBackground.height();
	int iWidth = theBackground.width();
	int iFormat = getFormat (iWidth, iHeight);
	if (iFormat == FORMAT_NONE) 	{
		int iReturn = QMessageBox::warning (NULL, QString ("Wrong IMAGE size"),
			QString ("This image is neither PAL, SECAM, nor NTSC. For a DVD the size has to fit either :\n"
			"Note, the DVD only supports:\n"
			"NTSC -> 720x480, 704x480, 352x480, 352x240 (29.97 Hz)\n"
			"PAL    -> 720x576, 704x576, 352x576, 352x288 (25 Hz)\n"
			"You need to resize this image sooner or later. Do you want to resize now ?"
			), QMessageBox::Yes ,  QMessageBox::No);
			if (iReturn == QMessageBox::Yes)	{
				CResizeDialog *pResizeDialog = new CResizeDialog ();
				pResizeDialog->setFilename(fileName);
				pResizeDialog->exec ();
				iWidth = pResizeDialog->m_pEditWidth->text().toInt();
				iHeight  = pResizeDialog->m_pEditHeight->text().toInt();
				iFormat = pResizeDialog->m_iFormat;
				theBackground.resize (iWidth, iHeight);
				QImage theFile = QImage (fileName, 0);
				theBackground.convertFromImage (theFile.smoothScale(iWidth, iHeight));
			}
	}
	m_pPixmapMenu->setFixedSize (iWidth, iHeight);
	m_pPixmapMenu->setPixmap ( (const QPixmap &) theBackground);
	// And finally we set the interface (to store the dvd project file in DVDAuthor class)
	m_DVDMenuInterface.qsBackgroundFileName = fileName;
	m_DVDMenuInterface.iFormat		= iFormat;
	m_DVDMenuInterface.iWidth		= iWidth;
	m_DVDMenuInterface.iHeight		= iHeight;
	m_DVDMenuInterface.bModified	= true;
}

#include "xml_dvdauthor.h"
void CDVDMenu::slotAddSound()
{
//	QMessageBox::warning (NULL, QString ("Hey header"),
//		QString ("Add Sound"), QMessageBox::Ok ,  QMessageBox::Cancel);
	CXmlDVDAuthor *pAuthor = new CXmlDVDAuthor ();
	pAuthor->readXml();
	pAuthor->writeXml();
}

void CDVDMenu::slotAddMask()
{
	QString fileName = QFileDialog::getOpenFileName ( QString("./"), QString ("Images (*.png *.xpm)"));

}

void CDVDMenu::slotDrawMask()
{
	QMessageBox::warning (NULL, QString ("Hey header"),
		QString ("Draw Mask / Button"), QMessageBox::Ok ,  QMessageBox::Cancel);
}

void CDVDMenu::slotZoomIn()
{
	QMessageBox::warning (NULL, QString ("Hey header"),
		QString ("ZoomIn"), QMessageBox::Ok ,  QMessageBox::Cancel);
}

void CDVDMenu::slotZoomOut()
{
	QMessageBox::warning (NULL, QString ("Hey header"),
		QString ("ZoomOut"), QMessageBox::Ok ,  QMessageBox::Cancel);
}

void CDVDMenu::slotAbout()
{
	QMessageBox::warning (NULL, QString ("Hey header"),
		QString ("About"), QMessageBox::Ok ,  QMessageBox::Cancel);
}

int CDVDMenu::getFormat (int iWidth, int iHeight)
{
	int iFormat = FORMAT_NONE;
	if ((iWidth == 720) && (iHeight == 480) )
		iFormat = FORMAT_NTSC1;
	else if  ((iWidth == 704) && (iHeight == 480) )
		iFormat = FORMAT_NTSC2;
	else if  ((iWidth == 352) && (iHeight == 480) )
		iFormat = FORMAT_NTSC3;
	else if  ((iWidth == 352) && (iHeight == 240) )
		iFormat = FORMAT_NTSC4;
	else if  ((iWidth == 720) && (iHeight == 576) )
		iFormat = FORMAT_PAL1;
	else if ((iWidth == 704) && (iHeight == 576) )
		iFormat = FORMAT_PAL2;
	else if  ((iWidth == 352) && (iHeight == 576) )
		iFormat = FORMAT_PAL3;
	else if  ((iWidth == 352) && (iHeight == 288) )
		iFormat = FORMAT_PAL4;
	return iFormat;
}

CDVDMenuInterface * CDVDMenu::getInterface ()
{
	return &m_DVDMenuInterface;
}

// This function is called when the user loads a new project file
void CDVDMenu::updateDVDMenu ()
{
	QFileInfo fileInfo (m_DVDMenuInterface.qsBackgroundFileName);
	if (!fileInfo.exists())	{
		QMessageBox::warning (NULL, QString ("File not found !"),
			QString ("Warning, could not find file %1.").arg(m_DVDMenuInterface.qsBackgroundFileName), QMessageBox::Ok ,  QMessageBox::Cancel);
		return;
	}
	// Create empty pixmap.
	QPixmap theBackground (m_DVDMenuInterface.iWidth, m_DVDMenuInterface.iHeight);
	// Load the image
	QImage theFile = QImage (m_DVDMenuInterface.qsBackgroundFileName, 0);
	theBackground.convertFromImage (theFile.smoothScale(m_DVDMenuInterface.iWidth, m_DVDMenuInterface.iHeight));

	m_pPixmapMenu->setFixedSize (m_DVDMenuInterface.iWidth, m_DVDMenuInterface.iHeight);
	m_pPixmapMenu->setPixmap ( (const QPixmap &) theBackground);
}


