/****************************************************************************
** Interface to the GUI.
**
** Created: Sun Dec 14 12:09:08 2003
**      by: Varol Okan using kate editor
**
** This class generates the command line for dvdauthor.
** Current Version of DVDAUTHOR used is 0.67
** You can find dvdauthor under http://dvdauthor.sourceforge.net/
**
****************************************************************************/

#ifndef MENU_H
#define MENU_H

#include <qobject.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <qaction.h>
#include <qdom.h>

#include "qdvdauthor.h"

class CDVDMenuInterface
{
public:
	  CDVDMenuInterface ();
	~CDVDMenuInterface ();

	void createXml (QDomDocument *pXmlDocument);
	bool readXml    (QDomDocument *pXmlDocument);

	// helper function ...
	void extractXml (QDomNode *pNode);

	// variables.
	bool bModified;
	QString qsBackgroundFileName;
	int iFormat;
	int iWidth;
	int iHeight;
};


class CDVDMenu : public QObject
{
	Q_OBJECT
	 CDVDMenu ();
public:
	// Constructor / destructor
	 CDVDMenu (QDVDAuthor *pDVDAuthor); 
	~CDVDMenu ();

public slots:
	virtual void slotAddBackground();
	virtual void slotAddSound();
	virtual void slotAddMask();
	virtual void slotDrawMask();
	virtual void slotZoomIn();
	virtual void slotZoomOut();
	virtual void slotAbout();

public:	// public functions
	CDVDMenuInterface * getInterface ();
	void updateDVDMenu();

private: // Privte functions ...
	void CreateTab (QTabWidget *pTabWidgetMain);
	void CreateToolBar (QDVDAuthor *pDVDAuthor);
	void CreateMenuEntry (QDVDAuthor *pDVDAuthor);

private:
	// helper functions ...
	int getFormat (int iWidth, int iHeight);

private:
	QWidget		*m_pDVDMenuTab;
	QGridLayout	*m_pTabLayout;
	QGridLayout	*m_pLayout1;
	QHBoxLayout	*m_pLayout2;
	QLabel			*m_pPixmapMenu;

	QPushButton	*m_pButtonAddBackground;
	QPushButton	*m_pButtonAddSound;
	QPushButton	*m_pButtonAddMask;
	QPushButton	*m_pButtonDrawMask;

	// Here are the private member actions ...
	QAction *m_pActionZoomIn;
	QAction *m_pActionZoomOut;
	QAction *m_pActionAbout;

	QToolBar *m_pToolBar;
	QPopupMenu *m_pDVDMenuMenu;
	
	CDVDMenuInterface m_DVDMenuInterface;
};

#endif // MENU_H
