/****************************************************************************
** qdvdauthor.h file.
**
** This file handles the user input to generate a DVD menu.
** Please note that this is work in progress and depends also oin the
** developement of dvdauthor, which is at the moment in version 0.67.
** Future versions of dvdauthort might need changes to this file.
**
*****************************************************************************/
#include <iostream>

#include <qmessagebox.h>
#include <qlineedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlistbox.h>

#include "global.h"
#include "xml_dvd.h"
#include "dialogsetup.h"
#include "qdvdauthor.h"
#include "dvdmenu.h"


QDVDAuthor::QDVDAuthor ()
{
	m_pDVDMenu = NULL;
	// The user did not give a project file name yet.
	m_bProjectNameInitialized = false;
	// Init the variables
	m_bModified = false;
	m_qsProjectFileName = QString (PROJECT_FILENAME);
	m_qsConvertPath 	= QString ("/usr/local/bin");
	m_qsDVDAuthorPath	= QString ("/usr/local/bin");
	m_qsMpegToolsPath	= QString ("/usr/local/bin");
	m_iNrOfMovieFiles	= 0;
	m_ppArrayMovieFiles = NULL;

	// Load the ini - file
	LoadIni ();

	m_pListBoxVideos->clear ();
	m_pListBoxVideos->insertItem( trUtf8( "<<Add Movie>>" ) );
	connect( m_pListBoxVideos, SIGNAL( clicked(QListBoxItem *) ), this, SLOT( slotAddFirstMovie(QListBoxItem *) ) );
}

QDVDAuthor::~QDVDAuthor ()
{
	SaveIni ();
	// Here we clean up after the array.
//	if (m_ppArrayMovieFiles)	{
//		for (int t=0;t<m_iNrOfMovieFiles;t++)
//			delete m_ppArrayMovieFiles[t];
//		delete []m_ppArrayMovieFiles;
//	}
}

void QDVDAuthor::addDVDMenu (CDVDMenu *pDVDMenu)
{
	m_pDVDMenu = pDVDMenu;
}

// Here are the standard callbacks .
void QDVDAuthor::slotSetup()
{
	CSetupDialog *pDialog = new CSetupDialog (this);

	// Here we set up the data of the Setup dialog ...	
	pDialog->m_pEditConvertPath->setText(m_qsConvertPath);
	pDialog->m_pEditDVDAuthorPath->setText(m_qsDVDAuthorPath);
	pDialog->m_pEditMpegToolsPath->setText(m_qsMpegToolsPath);
	
	pDialog->exec();
	
	m_qsConvertPath 	= pDialog->m_pEditConvertPath->text();
	m_qsDVDAuthorPath	= pDialog->m_pEditDVDAuthorPath->text();
	m_qsMpegToolsPath	= pDialog->m_pEditMpegToolsPath->text();
}

void QDVDAuthor::fileNew()
{

}

void QDVDAuthor::fileOpen()
{
	// First we check if there are modifications to be saved
	if ( (m_pDVDMenu->getInterface()->bModified) || (m_bModified) )	{
		int iReturn = QMessageBox::warning ( this, QString ("File not saved"),
			QString ("The current modifications have not been save to the file. \nDo you want to save the project now ?"),
			QMessageBox::Yes, QMessageBox::No);
		if (iReturn == QMessageBox::Yes)
			fileSave ();
	}
	// Now let the user pick a new project to open.
	QString qsProjectFileName = QFileDialog::getOpenFileName("./", "Project Files (*.xml)", this, "Open Project file dialog", "Open Project File");
	// Sanity check .. Did te user choose a new project file ?
	if (qsProjectFileName.isNull ())
		return;

	// Assign the file
	QFile projectFile(qsProjectFileName);
	if (!projectFile.open(IO_Raw | IO_ReadWrite))
		return;
	// Try to get the right xml contents ...
	QDomDocument xmlDoc( XML_DOCTYPE );
	if (!xmlDoc.setContent (&projectFile))	{
		// Error handling ...
		projectFile.close();
		int iReturn = QMessageBox::warning ( this, QString ("xml proect file seems to be defective."),
			QString ("Do you want to try to load another project file ?"),
			QMessageBox::Yes, QMessageBox::No);
		if (iReturn == QMessageBox::Yes)
			fileOpen ();	// Okay, user wants to specify another project file.
		return;
	}
	// And at last lets try to read the information of the file.
	if (!m_pDVDMenu->getInterface()->readXml (&xmlDoc))	{
		projectFile.close();
		return;
	}
	m_pDVDMenu->updateDVDMenu();

	projectFile.close();

	// Setting of the variables ...
	m_bModified = false;
	m_bProjectNameInitialized = true;
	m_qsProjectFileName = qsProjectFileName;
}

void QDVDAuthor::fileSave()
{
	if ( (m_pDVDMenu->getInterface()->bModified) || (m_bModified) )	{
		if (!m_bProjectNameInitialized)	{
			fileSaveAs ();
			return;
		}
		printf ("%s\n", (const char *)m_qsProjectFileName);

		QFile projectFile(m_qsProjectFileName);
		projectFile.open(IO_WriteOnly);

		QDomDocument xmlDoc( XML_DOCTYPE );
		QDomElement rootDVDMenu = xmlDoc.createElement( DVD_PROJECT );
		xmlDoc.appendChild( rootDVDMenu );
		m_pDVDMenu->getInterface()->createXml (&xmlDoc);

		QString xml = xmlDoc.toString();
		printf ("%s\n", (const char *)xml);
		projectFile.writeBlock(xml, qstrlen (xml));

		projectFile.close();
	}
	// re-set all modified flags ...
	m_bModified = false;
	m_pDVDMenu->getInterface()->bModified = false;
}

void QDVDAuthor::fileSaveAs()
{
	m_qsProjectFileName = QFileDialog::getSaveFileName("./", "Project Files (*.xml)", this, "Save file dialog", "Save File As ...");
	if (m_qsProjectFileName.isNull ())
		return;
	m_bModified = true;
	m_bProjectNameInitialized = true;
	QFileInfo fileInfo (m_qsProjectFileName);
	if (fileInfo.extension(FALSE).isEmpty())	// Indication there is no extension ...
		m_qsProjectFileName += QString (".xml");

	fileSave();
}

void QDVDAuthor::slotAddFirstMovie(QListBoxItem *pItem)
{
	// First we check if the item is valid ...
	if (pItem == NULL)	{
		slotEditAddMovie ();
		return;
	}
		// Then we check if the user clicked on the "<<Add Movie>> button...
	if (pItem->text () == QString ("<<Add Movie>>"))
		slotEditAddMovie ();
	// If neither of those two was the case, then we have to handle the movie ...
	// functionHandleMovie (pItem->text ());
}

void QDVDAuthor::slotExportDVDAuthorXML()
{

}

void QDVDAuthor::slotEditAddMovie ()
{
	// For now we assume thios is slotEditAdd()
	QString qsAddMovie = QFileDialog::getOpenFileName("./", "Movie Files (*.vob; *.mpg; *.mpeg; *.m2v)", this, "Save file dialog", "Add To Project ...");
	if (qsAddMovie.isNull())
		return;
	m_iNrOfMovieFiles ++;
	QString **ppNewArray = new QString *[m_iNrOfMovieFiles];
	for (int t=0; t<m_iNrOfMovieFiles-1; t++)	{
		ppNewArray[t] = m_ppArrayMovieFiles[t];
	}
	delete []m_ppArrayMovieFiles;
	QString *pNewMovieName = new QString(qsAddMovie);
	ppNewArray[m_iNrOfMovieFiles] = pNewMovieName;
	m_ppArrayMovieFiles = ppNewArray;

	// Okay, now we should also add this into the list box ...
	QFileInfo fileInfo (qsAddMovie);
	m_pListBoxVideos->insertItem( fileInfo.fileName(), m_pListBoxVideos->count ()-1 );
//	m_pListBoxVideos->disconnect(SIGNAL( clicked(QListBoxItem *)));
}

void QDVDAuthor::filePrint()
{

}

void QDVDAuthor::fileExit()
{

}

void QDVDAuthor::editUndo()
{

}

void QDVDAuthor::editRedo()
{

}

void QDVDAuthor::editCut()
{

}

void QDVDAuthor::editPaste()
{

}

void QDVDAuthor::editFind()
{

}

void QDVDAuthor::helpIndex()
{

}

void QDVDAuthor::helpContents()
{

}

void QDVDAuthor::helpAbout()
{

}

// Here are the standard aux functions
void QDVDAuthor::LoadIni ()
{
	// the fileName of the INI file is a #define in global.h
	QFile file (INI_FILE_NAME);
}

void QDVDAuthor::SaveIni ()
{
	
}


